/* -*- mode: c++; c-basic-offset: 4; -*- */
#ifndef _LIGOWEBCLIENT_H
#define _LIGOWEBCLIENT_H
/*----------------------------------------------------------------------*/
/*                                                         		*/
/* Module Name: webclient						*/
/*                                                         		*/
/* Module Description: Class to access a web server			*/
/*                                                         		*/
/* Revision History:					   		*/
/* Rel   Date     Programmer  	Comments				*/
/* 0.1	 23Feb05  D. Sigg    	First release		   		*/
/*                                                         		*/
/* Documentation References:						*/
/*	Man Pages: webclient.html					*/
/*	References: none						*/
/*                                                         		*/
/* Author Information:							*/
/* Name          Telephone       Fax             e-mail 		*/
/* Daniel Sigg   (509) 372-8132  (509) 372-8137  sigg_d@ligo.mit.edu	*/
/*                                                         		*/
/*                                                         		*/
/*                      -------------------                             */
/*                                                         		*/
/*                             LIGO					*/
/*                                                         		*/
/*        THE LASER INTERFEROMETER GRAVITATIONAL WAVE OBSERVATORY.	*/
/*                                                         		*/
/*                     (C) The LIGO Project, 1999.			*/
/*                                                         		*/
/*                                                         		*/
/* Caltech				MIT		   		*/
/* LIGO Project MS 51-33		LIGO Project NW-17 161		*/
/* Pasadena CA 91125			Cambridge MA 01239 		*/
/*                                                         		*/
/* LIGO Hanford Observatory		LIGO Livingston Observatory	*/
/* P.O. Box 1970 S9-02			19100 LIGO Lane Rd.		*/
/* Richland WA 99352			Livingston, LA 70754		*/
/*                                                         		*/
/*----------------------------------------------------------------------*/

#include <string>
#include <map>

namespace web {

/** Class to access a web server.
   
    @memo web server access class
    @author Written February 2005 by Daniel Sigg
    @version 1.0
    @ingroup IO_web
 ************************************************************************/
   class http_request {
   public:
      /// Header sort order
      class header_sort {
      public:
         /// ignore case
         bool operator () (const std::string& s1, const std::string& s2) const;
      };
      /// header type (name/token)
      typedef std::map <std::string, std::string, header_sort> header_type;
   
      /** Default constructor.
          @memo Default constructor
       ******************************************************************/
      http_request ();
      /** Constructor.
          Open connection, send request and wait for answer.
          @memo Open connection, send request and wait for answer
          @param url URL string
          @param addr Web server address
          @param port Web server port number
   	  @return true if successful
       ******************************************************************/
      http_request (const char* url, const char* addr, int port = 80);
      /** Destructor.
          @memo Destructor
       ******************************************************************/
      ~http_request ();
      /** Open connection.
          @memo Open connection
          @param addr Web server address
          @param port Web server port number
   	  @return true if successful
       ******************************************************************/
      bool open (const char* addr, int port = 80);
      /** Close connection.
          @memo Close connection
       ******************************************************************/
      void close();
      /// send request and wait for answer
      /** Send request and wait for answer.
          @memo Send request and wait for answer
          @param url URL string
   	  @return true if successful
       ******************************************************************/
      bool request (const char* url);
      /** Open connection, send request and wait for answer.
          @memo Open connection, send request and wait for answer
          @param url URL string
          @param addr Web server address
          @param port Web server port number
   	  @return true if successful
       ******************************************************************/
      bool request (const char* url, const char* addr, int port = 80);
   
      /// Get header
      const header_type& header() const {
         return fHeader; }
      /// Get content length
      int size() const {
         return fLen; }
      /// Get content pointer
      char* data() {
         return fData; }
   
      /** Mangle a string to escape control characters.
          @memo Mangle
          @param s input string
   	  @return mangled string
       ******************************************************************/
      static std::string mangle (const std::string& s);
      /** Demangle a string to unescape control characters.
          @memo Demangle
          @param s input string
   	  @return demangled string
       ******************************************************************/
      static std::string demangle (const std::string& s);
   
   protected:
      /// header
      header_type	fHeader;
      /// Data
      char*		fData;
      /// Data length
      int		fLen;
      /// filled data
      int		fFill;
      /// Socket
      int		fSock;
   
      /// send an http request
      bool sendRequest (const char* url);
      /// receive the header of the response
      bool recvHeader();
      /// receive the content
      bool recvContent();
      /// uncompress the content
      bool uncompress();
   
   private:
      http_request (const http_request&);
      http_request& operator= (const http_request&);
   };


}

#endif
